/**
 * <p>Title: OWAMP Protocol</p>
 * <p>Description: This project contains the classes comum to Sender and Receiver.</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: University of Aveiro / Institute of Telecommunications - Aveiro pole</p>
 * @authors: Helder Veiga and Teresa Pinho
 * @version: 1.2
 * @Last updated: 9-12-2005
 *
 * Copyright (C) 2005  Helder Veiga and Teresa Pinho
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.  A HTML version of
 * the GNU General Public License can be seen at
 * http://www.gnu.org/licenses/gpl.html.
 *
 */


import java.net.*;

public class OWAMP_SessionTerminal extends OWAMP_Base{

  // Constructors

  public OWAMP_SessionTerminal(int[] portRange) {
    this.portRange = portRange;
  }

  // Instance variables

  protected Socket socketToCS = new Socket();                   // Represents the socket connection to the Client or Server.
  protected RequestSessionReceiver dataRS;                      // Represents the RequestSession datagram.
  protected ScheduleSlotDescriptionsReceiver[] arrayOfdataSSD;  // Represents the arrayOfdataSSD of the RequestSession datagram
                                                                // that is going to be processed.
  protected StopSessionsReceiver dataSS;                        // Represents the StopSessions datagram.
  protected SessionPacketsSentRecordReceiver[] arrayOfdataSPSR; // Represents the SessionPacketsSentRecord datagram.
  protected DatagramSocket ds;                                  // The DatagramSocket for UDP Socket.
  protected String element;                                     // Represents the element in use (Sender or Receiver).
  protected int[] portRange = null;                             // Represents the UDP port range to be used during test sessions.


  // Instance methods

  /**
   * Method to receive the packet Request Session send by a client.
   * To each Request-Session message, an OWAMP Session-Receiver MUST respond with an Accept-Session message.
   * @param accept short - specify if the Session-Receiver accepts the request. In this message, zero in the
   * Accept field means that the Session-Receiver is willing to conduct the session.  A value of 1 indicates
   * rejection of the request.  All other values are reserved.
   * @return boolean - This method returns true if the request operation was ok, otherwise it returns false.
   */

  public boolean requestSessionResponse(byte accept){
    return true;
  }


  /**
   * Having requested one or more test sessions and received affirmative Accept-Session responses, an OWAMP
   * client may start the execution of the requested test sessions by sending a Start-Sessions message to
   * the server.
   * @param accept byte - If Accept is 1, the Start-Sessions request was rejected. Zero means that the
   * command was accepted.  All other values are reserved.  The server MAY and the client SHOULD close the
   * connection in the case of a negative response.
   * @return boolean -  It returns true if everything goes ok, otherwise it returns false.
   */

  public boolean startingTestSessionsResponse(byte accept){ // process Starting Test Sessions Response
    return true;
  }


  /**
   * This method is used to determine which packet was received and process it.
   * The first byte read represents the type of the packet.
   * @return boolean - It returns true if everything goes ok, otherwise it returns false.
   */

  public boolean processPacketReceived(){

    int readn = 0;  // number of bytes read

    switch((int)(buffer[0] & 0xFF)){
      case 1:  // Request-Session
        readn = receivePacket(socketToCS, buffer, 1, RequestSession.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (RequestSession.PACKET_LENGTH -1)){   // Packet Request Session
          System.out.println("Request Session from "+element+" "+socketToCS);
          System.out.println();

          byte accept = 0;

          if (!requestSessionResponse(accept)) { // requestSession(byte accept)
            System.err.println("Error: Error in Request-Session operation.");
            return false;
          }

        }else{
          System.err.println("Error: Error in Request-Session operation.");
          return false;
        }
        return true;
      case 2:  // Starting Test Sessions
        readn = receivePacket(socketToCS,buffer,1,StartingTestSessions.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (StartingTestSessions.PACKET_LENGTH -1)){   // Packet Starting Test Sessions
          System.out.println("Starting Test Sessions from "+element+" "+socketToCS);
          System.out.println();

          byte accept = 0;
          if (!startingTestSessionsResponse(accept)) {  // startingTestSessionsResponse(byte accept)
            System.err.println("Error: Error in Starting Test Sessions operation.");
            return false;
          }
          return true;
        }
        return false;
      case 3:  // Stop Sessions
        readn = receivePacket(socketToCS,buffer,1,StopSessions.PACKET_LENGTH -1);   // receivePacket (Socket socket, byte[] _buffer, int off, int len)
        if(readn == (StopSessions.PACKET_LENGTH -1)){   // Packet Stop Sessions

          System.out.println("Stop Sessions from "+element+" "+socketToCS);
          System.out.println();

          // In variable byte[] buffer we have the packet stopSessions received

          // Create the stopSessions packet

          dataSS = new StopSessionsReceiver();

          System.arraycopy(buffer,0,dataSS.buffer,0,dataSS.PACKET_LENGTH);

          // Initialize the list to receive the SessionPacketsSentRecord packets.

          int numberOfSessions = Packets.getIntFromByteArray(buffer,4);  // number Of SessionPacketsSentRecord to receive
          arrayOfdataSPSR = new SessionPacketsSentRecordReceiver[numberOfSessions];
          arrayOfdataSPSR = dataSS.receiveCompleteSS(socketToCS);  // Receive complete stopSessions

          if (arrayOfdataSPSR == null) { // Receive complete stopSessions
            if(dataSS.accept != 1){
              System.err.println("Error: Error in Stop Sessions operation.");
              System.err.println("Error: Packet received is not a Stop Session packet.");
              closeSocket(socketToCS);
              return false;
            }

            System.err.println("Error: Stop Session, with  accept = 1, received from "+element+": "+socketToCS);
            System.err.println("Error: This session will be finished.");
          }

          closeSocket(socketToCS);
          return true;
        }


        return false;
      case 4:   // Fetch-Session
        if(!processFetchSession()){
         System.err.println("Error: Error processing Fetch session data operation.");
         closeSocket(socketToCS);
         return false;
       }else
         return true;
      default:
       System.err.println("Error: Error in Request-Session operation. Length of packet received doesn't match.");
    }

    return false;

  }


  /**
   * This method is used, by the Receiver, to process the Fetch Session.
   */

  public boolean processFetchSession(){
    return true;
  }

}
